type 'a eqclass' = Pointer of 'a eqclass' ref | Element of ('a option * Util.IntSet.t);;
type 'a eqclass = 'a eqclass' ref
type 'a eqclasses = ('a eqclass) Util.IntMap.t;;

exception Fatal of string;;

let empty:'a eqclasses = Util.IntMap.empty;;

let rec find_element p =
  match !p with
    Element _ -> p
  | Pointer p' -> find_element p'

let find m classes = find_element(Util.IntMap.find m classes);;
let find_and_add m classes =
   try
     (find m classes, classes)
   with Not_found ->
     let c = ref (Element(None, Util.IntSet.singleton m)) in
      (c, (Util.IntMap.add m c classes))


let representative p = 
  match !(find_element p) with
    Element(r,_) -> r
   | _ -> raise (Fatal "representative");;

let elements_of_class p =
  match !(find_element p) with
    Element(_,s) -> s
   | _ -> raise (Fatal "elements_of_class");;


let choose_from_class p =
  match !(find_element p) with
    Element(_,s) -> Util.IntSet.choose s
   | _ -> raise (Fatal "choose_from_class");;

let merge classp1 classp2 classes =
  let p1 = find_element(classp1) in
  let p2 = find_element(classp2) in
    match(!p1, !p2) with
      (Element(r1,elements1), Element(r2,elements2)) ->
         let elements = Util.IntSet.union elements1 elements2 in
           if r1=None then (p2:=Element(r2,elements); p1 := Pointer(p2); classes)
           else 
              (p1:=Element(r1,elements); p2:=Pointer(p1); classes)
     | _ -> raise (Fatal "merge");;

let eqclass class1 class2 = (find_element class1) == (find_element class2);;

let add_representative r classp classes =
  let p' = find_element classp in
   match !p' with
     Element(_,elements) -> (p' := Element(Some r,elements); classes)
   | _ -> raise (Fatal "add_representative");;

let sets_of_elements classes =
  Util.IntMap.fold (function k -> function cl -> Util.IntSet.add k) classes Util.IntSet.empty;;

