(*** Module for simple type inference ***)

(*** The identifier for a recursive type:
 *** Currently, only list types (whose identifier is 0) are supported
 ***)
type rectyId = int

(*** Identifers for type variables ***)
type tvid = int

(*** Simple types ***)
type stype = TVar of tvid
           | UnitT
           | BoolT
           | IntT
           | ChanT of stype
           | PairT of stype * stype
           | FunT of stype * stype
           | RecT of rectyId * stype list 

exception Unif of stype * stype

(*** Type environments ***)
type tenv = (Syntax.vid, stype) Utilities.env

(*** Type inference:
 *** Given a process expression, return a pair consisting of
 ***   the process whose bound variables are annotated with simple types, and
 ***   the type environment
 ***)
val elaborate: unit Syntax.procexp -> stype Syntax.procexp * tenv

(*** Returns the list of the processes enclosed by (#T ... #) ***)
val get_specials: stype Syntax.procexp -> tenv -> (stype Syntax.procexp * tenv) list
(*** 
 *** for printing 
 ***)

(*** given a type, return its string representation ***)
val string_of_type: stype -> string
(*** print type environment ***)
val print_env: tenv -> unit

